@@ -1,6 +1,6 @@
 /*
  * Custom types to store LV2 information
- * Copyright (C) 2011-2019 Filipe Coelho <falktx@falktx.com>
+ * Copyright (C) 2011-2022 Filipe Coelho <falktx@falktx.com>
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -96,6 +96,7 @@ typedef uint32_t LV2_Property;
 #define LV2_PORT_UNIT_PC                 22
 #define LV2_PORT_UNIT_S                  23
 #define LV2_PORT_UNIT_SEMITONE           24
+#define LV2_PORT_UNIT_VOLTS              25
 
 #define LV2_IS_PORT_UNIT_BAR(x)          ((x) == LV2_PORT_UNIT_BAR)
 #define LV2_IS_PORT_UNIT_BEAT(x)         ((x) == LV2_PORT_UNIT_BEAT)
@@ -121,6 +122,7 @@ typedef uint32_t LV2_Property;
 #define LV2_IS_PORT_UNIT_PC(x)           ((x) == LV2_PORT_UNIT_PC)
 #define LV2_IS_PORT_UNIT_S(x)            ((x) == LV2_PORT_UNIT_S)
 #define LV2_IS_PORT_UNIT_SEMITONE(x)     ((x) == LV2_PORT_UNIT_SEMITONE)
+#define LV2_IS_PORT_UNIT_VOLTS(x)        ((x) == LV2_PORT_UNIT_VOLTS)
 
 // Port Types
 #define LV2_PORT_INPUT                   0x001
@@ -378,7 +380,7 @@ struct LV2_RDF_PortUnit {
         }
     }
 
-    CARLA_DECLARE_NON_COPY_STRUCT(LV2_RDF_PortUnit)
+    CARLA_DECLARE_NON_COPYABLE(LV2_RDF_PortUnit)
 };
 
 // Port Scale Point
@@ -399,7 +401,7 @@ struct LV2_RDF_PortScalePoint {
         }
     }
 
-    CARLA_DECLARE_NON_COPY_STRUCT(LV2_RDF_PortScalePoint)
+    CARLA_DECLARE_NON_COPYABLE(LV2_RDF_PortScalePoint)
 };
 
 // Port
@@ -465,7 +467,7 @@ struct LV2_RDF_Port {
         }
     }
 
-    CARLA_DECLARE_NON_COPY_STRUCT(LV2_RDF_Port)
+    CARLA_DECLARE_NON_COPYABLE(LV2_RDF_Port)
 };
 
 // Port
@@ -493,7 +495,7 @@ struct LV2_RDF_PortGroup {
         }
     }
 
-    CARLA_DECLARE_NON_COPY_STRUCT(LV2_RDF_PortGroup)
+    CARLA_DECLARE_NON_COPYABLE(LV2_RDF_PortGroup)
 };
 
 // Parameter
@@ -568,7 +570,7 @@ struct LV2_RDF_Parameter {
         other.Unit.Symbol = nullptr;
     }
 
-    CARLA_DECLARE_NON_COPY_STRUCT(LV2_RDF_Parameter)
+    CARLA_DECLARE_NON_COPYABLE(LV2_RDF_Parameter)
 };
 
 // Preset
@@ -594,7 +596,7 @@ struct LV2_RDF_Preset {
         }
     }
 
-    CARLA_DECLARE_NON_COPY_STRUCT(LV2_RDF_Preset)
+    CARLA_DECLARE_NON_COPYABLE(LV2_RDF_Preset)
 };
 
 // Feature
@@ -615,7 +617,7 @@ struct LV2_RDF_Feature {
         }
     }
 
-    CARLA_DECLARE_NON_COPY_STRUCT(LV2_RDF_Feature)
+    CARLA_DECLARE_NON_COPYABLE(LV2_RDF_Feature)
 };
 
 // Port Notification
@@ -638,7 +640,7 @@ struct LV2_RDF_UI_PortNotification {
         }
     }
 
-    CARLA_DECLARE_NON_COPY_STRUCT(LV2_RDF_UI_PortNotification)
+    CARLA_DECLARE_NON_COPYABLE(LV2_RDF_UI_PortNotification)
 };
 
 // UI
@@ -711,7 +713,7 @@ struct LV2_RDF_UI {
         }
     }
 
-    CARLA_DECLARE_NON_COPY_STRUCT(LV2_RDF_UI)
+    CARLA_DECLARE_NON_COPYABLE(LV2_RDF_UI)
 };
 
 // Plugin Descriptor
@@ -849,7 +851,7 @@ struct LV2_RDF_Descriptor {
         }
     }
 
-    CARLA_DECLARE_NON_COPY_STRUCT(LV2_RDF_Descriptor)
+    CARLA_DECLARE_NON_COPYABLE(LV2_RDF_Descriptor)
 };
 
 #endif // LV2_RDF_HPP_INCLUDED
