@@ -2,7 +2,7 @@
   ==============================================================================
 
    This file is part of the JUCE library.
-   Copyright (c) 2020 - Raw Material Software Limited
+   Copyright (c) 2022 - Raw Material Software Limited
 
    JUCE is an open source library subject to commercial or open-source
    licensing.
@@ -23,6 +23,8 @@
 namespace juce
 {
 
+JUCE_BEGIN_IGNORE_WARNINGS_GCC_LIKE ("-Wlanguage-extension-token")
+
 #ifndef JUCE_WASAPI_LOGGING
  #define JUCE_WASAPI_LOGGING 0
 #endif
@@ -31,7 +33,7 @@ namespace juce
 namespace WasapiClasses
 {
 
-void logFailure (HRESULT hr)
+static void logFailure (HRESULT hr)
 {
     ignoreUnused (hr);
     jassert (hr != (HRESULT) 0x800401f0); // If you hit this, it means you're trying to call from
@@ -87,7 +89,7 @@ void logFailure (HRESULT hr)
 
 #undef check
 
-bool check (HRESULT hr)
+static bool check (HRESULT hr)
 {
     logFailure (hr);
     return SUCCEEDED (hr);
@@ -116,9 +118,6 @@ bool check (HRESULT hr)
  #define KSDATAFORMAT_SUBTYPE_IEEE_FLOAT  uuidFromString ("00000003-0000-0010-8000-00aa00389b71")
 #endif
 
-#define JUCE_IUNKNOWNCLASS(name, guid)   JUCE_COMCLASS(name, guid) : public IUnknown
-#define JUCE_COMCALL                     virtual HRESULT STDMETHODCALLTYPE
-
 enum EDataFlow
 {
     eRender = 0,
@@ -337,15 +336,32 @@ JUCE_IUNKNOWNCLASS (IAudioSessionControl, "F4B1A599-7266-4319-A8CA-E70ACB11E8CD"
     JUCE_COMCALL UnregisterAudioSessionNotification (IAudioSessionEvents*) = 0;
 };
 
-#undef JUCE_COMCALL
-#undef JUCE_COMCLASS
-#undef JUCE_IUNKNOWNCLASS
+} // namespace juce
+
+#ifdef __CRT_UUID_DECL
+__CRT_UUID_DECL (juce::IPropertyStore,        0x886d8eeb, 0x8cf2, 0x4446, 0x8d, 0x02, 0xcd, 0xba, 0x1d, 0xbd, 0xcf, 0x99)
+__CRT_UUID_DECL (juce::IMMDevice,             0xD666063F, 0x1587, 0x4E43, 0x81, 0xF1, 0xB9, 0x48, 0xE8, 0x07, 0x36, 0x3F)
+__CRT_UUID_DECL (juce::IMMEndpoint,           0x1BE09788, 0x6894, 0x4089, 0x85, 0x86, 0x9A, 0x2A, 0x6C, 0x26, 0x5A, 0xC5)
+__CRT_UUID_DECL (juce::IMMNotificationClient, 0x7991EEC9, 0x7E89, 0x4D85, 0x83, 0x90, 0x6C, 0x70, 0x3C, 0xEC, 0x60, 0xC0)
+__CRT_UUID_DECL (juce::IMMDeviceEnumerator,   0xA95664D2, 0x9614, 0x4F35, 0xA7, 0x46, 0xDE, 0x8D, 0xB6, 0x36, 0x17, 0xE6)
+__CRT_UUID_DECL (juce::MMDeviceEnumerator,    0xBCDE0395, 0xE52F, 0x467C, 0x8E, 0x3D, 0xC4, 0x57, 0x92, 0x91, 0x69, 0x2E)
+__CRT_UUID_DECL (juce::IAudioClient,          0x1CB9AD4C, 0xDBFA, 0x4c32, 0xB1, 0x78, 0xC2, 0xF5, 0x68, 0xA7, 0x03, 0xB2)
+__CRT_UUID_DECL (juce::IAudioClient2,         0x726778CD, 0xF60A, 0x4eda, 0x82, 0xDE, 0xE4, 0x76, 0x10, 0xCD, 0x78, 0xAA)
+__CRT_UUID_DECL (juce::IAudioClient3,         0x1CB9AD4C, 0xDBFA, 0x4c32, 0xB1, 0x78, 0xC2, 0xF5, 0x68, 0xA7, 0x03, 0xB2)
+__CRT_UUID_DECL (juce::IAudioCaptureClient,   0xC8ADBD64, 0xE71E, 0x48a0, 0xA4, 0xDE, 0x18, 0x5C, 0x39, 0x5C, 0xD3, 0x17)
+__CRT_UUID_DECL (juce::IAudioRenderClient,    0xF294ACFC, 0x3146, 0x4483, 0xA7, 0xBF, 0xAD, 0xDC, 0xA7, 0xC2, 0x60, 0xE2)
+__CRT_UUID_DECL (juce::IAudioEndpointVolume,  0x5CDF2C82, 0x841E, 0x4546, 0x97, 0x22, 0x0C, 0xF7, 0x40, 0x78, 0x22, 0x9A)
+__CRT_UUID_DECL (juce::IAudioSessionEvents,   0x24918ACC, 0x64B3, 0x37C1, 0x8C, 0xA9, 0x74, 0xA6, 0x6E, 0x99, 0x57, 0xA8)
+__CRT_UUID_DECL (juce::IAudioSessionControl,  0xF4B1A599, 0x7266, 0x4319, 0xA8, 0xCA, 0xE7, 0x0A, 0xCB, 0x11, 0xE8, 0xCD)
+#endif
 
 //==============================================================================
+namespace juce
+{
 namespace WasapiClasses
 {
 
-String getDeviceID (IMMDevice* device)
+static String getDeviceID (IMMDevice* device)
 {
     String s;
     WCHAR* deviceId = nullptr;
@@ -639,10 +655,10 @@ private:
                                                                     &minPeriod,
                                                                     &maxPeriod)))
                 {
-                    minBufferSize = minPeriod;
-                    defaultBufferSize = defaultPeriod;
-                    lowLatencyMaxBufferSize = maxPeriod;
-                    lowLatencyBufferSizeMultiple = fundamentalPeriod;
+                    minBufferSize = (int) minPeriod;
+                    defaultBufferSize = (int) defaultPeriod;
+                    lowLatencyMaxBufferSize = (int) maxPeriod;
+                    lowLatencyBufferSizeMultiple = (int) fundamentalPeriod;
                 }
             }
         }
@@ -660,9 +676,7 @@ private:
 
     void querySupportedSampleRates (WAVEFORMATEXTENSIBLE format, ComSmartPtr<IAudioClient>& audioClient)
     {
-        for (auto rate : { 8000, 11025, 16000, 22050, 32000,
-                           44100, 48000, 88200, 96000, 176400,
-                           192000, 352800, 384000, 705600, 768000 })
+        for (auto rate : SampleRateHelpers::getAllSampleRates())
         {
             if (rates.contains (rate))
                 continue;
@@ -679,7 +693,7 @@ private:
                                                                                         : &nearestFormat)))
             {
                 if (nearestFormat != nullptr)
-                    rate = nearestFormat->nSamplesPerSec;
+                    rate = (double) nearestFormat->nSamplesPerSec;
 
                 if (! rates.contains (rate))
                     rates.addUsingDefaultSort (rate);
@@ -787,7 +801,7 @@ private:
     {
         if (auto audioClient3 = client.getInterface<IAudioClient3>())
             return check (audioClient3->InitializeSharedAudioStream (getStreamFlags(),
-                                                                     bufferSizeSamples,
+                                                                     (UINT32) bufferSizeSamples,
                                                                      (WAVEFORMATEX*) &format,
                                                                      nullptr));
 
@@ -829,7 +843,7 @@ private:
             client = nullptr;
             client = createClient();
 
-            defaultPeriod = samplesToRefTime (numFrames, format.Format.nSamplesPerSec);
+            defaultPeriod = samplesToRefTime ((int) numFrames, format.Format.nSamplesPerSec);
         }
 
         return false;
@@ -872,7 +886,7 @@ public:
     {
     }
 
-    ~WASAPIInputDevice()
+    ~WASAPIInputDevice() override
     {
         close();
     }
@@ -889,11 +903,10 @@ public:
         closeClient();
         captureClient = nullptr;
         reservoir.reset();
-        reservoirReadPos = 0;
-        reservoirWritePos = 0;
+        queue = SingleThreadedAbstractFifo();
     }
 
-    template<class SourceType>
+    template <class SourceType>
     void updateFormatWithType (SourceType*) noexcept
     {
         using NativeType = AudioData::Pointer<AudioData::Float32, AudioData::NativeEndian, AudioData::NonInterleaved, AudioData::NonConst>;
@@ -908,13 +921,12 @@ public:
         else                            updateFormatWithType ((AudioData::Int16*)   nullptr);
     }
 
-    bool start (int userBufferSize)
+    bool start (int userBufferSizeIn)
     {
-        reservoirSize = actualBufferSize + userBufferSize;
-        reservoirMask = nextPowerOfTwo (reservoirSize) - 1;
-        reservoir.setSize ((reservoirMask + 1) * bytesPerFrame, true);
-        reservoirReadPos = 0;
-        reservoirWritePos = 0;
+        const auto reservoirSize = nextPowerOfTwo ((int) (actualBufferSize + (UINT32) userBufferSizeIn));
+
+        queue = SingleThreadedAbstractFifo (reservoirSize);
+        reservoir.setSize ((size_t) (queue.getSize() * bytesPerFrame), true);
         xruns = 0;
 
         if (! check (client->Start()))
@@ -932,93 +944,80 @@ public:
         UINT32 numSamplesAvailable;
         DWORD flags;
 
-        while (captureClient->GetBuffer (&inputData, &numSamplesAvailable, &flags, nullptr, nullptr)
-                  != MAKE_HRESULT (0, 0x889, 0x1) /* AUDCLNT_S_BUFFER_EMPTY */)
+        while (captureClient->GetBuffer (&inputData, &numSamplesAvailable, &flags, nullptr, nullptr) != MAKE_HRESULT (0, 0x889, 0x1) /* AUDCLNT_S_BUFFER_EMPTY */)
             captureClient->ReleaseBuffer (numSamplesAvailable);
     }
 
-    int getNumSamplesInReservoir() const noexcept    { return reservoirWritePos.load() - reservoirReadPos.load(); }
+    int getNumSamplesInReservoir() const noexcept    { return queue.getNumReadable(); }
 
     void handleDeviceBuffer()
     {
         if (numChannels <= 0)
             return;
 
-        uint8* inputData;
-        UINT32 numSamplesAvailable;
-        DWORD flags;
+        uint8* inputData = nullptr;
+        UINT32 numSamplesAvailable = 0;
+        DWORD flags = 0;
 
         while (check (captureClient->GetBuffer (&inputData, &numSamplesAvailable, &flags, nullptr, nullptr)) && numSamplesAvailable > 0)
         {
             if ((flags & AUDCLNT_BUFFERFLAGS_DATA_DISCONTINUITY) != 0)
                 xruns++;
 
-            int samplesLeft = (int) numSamplesAvailable;
-
-            while (samplesLeft > 0)
+            if (numSamplesAvailable > (UINT32) queue.getRemainingSpace())
             {
-                auto localWrite = reservoirWritePos.load() & reservoirMask;
-                auto samplesToDo = jmin (samplesLeft, reservoirMask + 1 - localWrite);
-                auto samplesToDoBytes = samplesToDo * bytesPerFrame;
-
-                void* reservoirPtr = addBytesToPointer (reservoir.getData(), localWrite * bytesPerFrame);
-
-                if ((flags & AUDCLNT_BUFFERFLAGS_SILENT) != 0)
-                    zeromem (reservoirPtr, samplesToDoBytes);
-                else
-                    memcpy (reservoirPtr, inputData, samplesToDoBytes);
-
-                reservoirWritePos += samplesToDo;
-                inputData += samplesToDoBytes;
-                samplesLeft -= samplesToDo;
+                captureClient->ReleaseBuffer (0);
+                return;
             }
 
-            if (getNumSamplesInReservoir() > reservoirSize)
-                reservoirReadPos = reservoirWritePos.load() - reservoirSize;
+            auto offset = 0;
+
+            for (const auto& block : queue.write ((int) numSamplesAvailable))
+            {
+                const auto samplesToDoBytes = block.getLength() * bytesPerFrame;
+
+                auto* reservoirPtr = addBytesToPointer (reservoir.getData(), block.getStart() * bytesPerFrame);
+
+                if ((flags & AUDCLNT_BUFFERFLAGS_SILENT) != 0)
+                    zeromem (reservoirPtr, (size_t) samplesToDoBytes);
+                else
+                    memcpy (reservoirPtr, inputData + offset * bytesPerFrame, (size_t) samplesToDoBytes);
+
+                offset += block.getLength();
+            }
 
             captureClient->ReleaseBuffer (numSamplesAvailable);
         }
     }
 
-    void copyBuffersFromReservoir (float** destBuffers, int numDestBuffers, int bufferSize)
+    void copyBuffersFromReservoir (float* const* destBuffers, const int numDestBuffers, const int bufferSize)
     {
-        if ((numChannels <= 0 && bufferSize == 0) || reservoir.getSize() == 0)
+        if ((numChannels <= 0 && bufferSize == 0) || reservoir.isEmpty())
             return;
 
-        int offset = jmax (0, bufferSize - getNumSamplesInReservoir());
+        auto offset = jmax (0, bufferSize - queue.getNumReadable());
 
         if (offset > 0)
-        {
             for (int i = 0; i < numDestBuffers; ++i)
-                zeromem (destBuffers[i], offset * sizeof (float));
+                zeromem (destBuffers[i], (size_t) offset * sizeof (float));
 
-            bufferSize -= offset;
-            reservoirReadPos -= offset / 2;
-        }
-
-        while (bufferSize > 0)
+        for (const auto& block : queue.read (jmin (queue.getNumReadable(), bufferSize)))
         {
-            auto localRead = reservoirReadPos.load() & reservoirMask;
-            auto samplesToDo = jmin (bufferSize, getNumSamplesInReservoir(), reservoirMask + 1 - localRead);
+            for (auto i = 0; i < numDestBuffers; ++i)
+                converter->convertSamples (destBuffers[i] + offset,
+                                           0,
+                                           addBytesToPointer (reservoir.getData(), block.getStart() * bytesPerFrame),
+                                           channelMaps.getUnchecked (i),
+                                           block.getLength());
 
-            if (samplesToDo <= 0)
-                break;
-
-            auto reservoirOffset = localRead * bytesPerFrame;
-
-            for (int i = 0; i < numDestBuffers; ++i)
-                converter->convertSamples (destBuffers[i] + offset, 0, addBytesToPointer (reservoir.getData(), reservoirOffset), channelMaps.getUnchecked(i), samplesToDo);
-
-            bufferSize -= samplesToDo;
-            offset += samplesToDo;
-            reservoirReadPos += samplesToDo;
+            offset += block.getLength();
         }
     }
 
     ComSmartPtr<IAudioCaptureClient> captureClient;
     MemoryBlock reservoir;
-    int reservoirSize, reservoirMask, xruns;
-    std::atomic<int> reservoirReadPos, reservoirWritePos;
+    SingleThreadedAbstractFifo queue;
+    int xruns = 0;
 
     std::unique_ptr<AudioData::Converter> converter;
 
@@ -1035,7 +1034,7 @@ public:
     {
     }
 
-    ~WASAPIOutputDevice()
+    ~WASAPIOutputDevice() override
     {
         close();
     }
@@ -1053,7 +1052,7 @@ public:
         renderClient = nullptr;
     }
 
-    template<class DestType>
+    template <class DestType>
     void updateFormatWithType (DestType*)
     {
         using NativeType = AudioData::Pointer<AudioData::Float32, AudioData::NativeEndian, AudioData::NonInterleaved, AudioData::Const>;
@@ -1073,8 +1072,8 @@ public:
         auto samplesToDo = getNumSamplesAvailableToCopy();
         uint8* outputData;
 
-        if (check (renderClient->GetBuffer (samplesToDo, &outputData)))
-            renderClient->ReleaseBuffer (samplesToDo, AUDCLNT_BUFFERFLAGS_SILENT);
+        if (check (renderClient->GetBuffer ((UINT32) samplesToDo, &outputData)))
+            renderClient->ReleaseBuffer ((UINT32) samplesToDo, AUDCLNT_BUFFERFLAGS_SILENT);
 
         if (! check (client->Start()))
             return false;
@@ -1094,13 +1093,13 @@ public:
             UINT32 padding = 0;
 
             if (check (client->GetCurrentPadding (&padding)))
-                return actualBufferSize - (int) padding;
+                return (int) actualBufferSize - (int) padding;
         }
 
-        return actualBufferSize;
+        return (int) actualBufferSize;
     }
 
-    void copyBuffers (const float** srcBuffers, int numSrcBuffers, int bufferSize,
+    void copyBuffers (const float* const* srcBuffers, int numSrcBuffers, int bufferSize,
                       WASAPIInputDevice* inputDevice, Thread& thread)
     {
         if (numChannels <= 0)
@@ -1157,11 +1156,11 @@ class WASAPIAudioIODevice  : public AudioIODevice,
 {
 public:
     WASAPIAudioIODevice (const String& deviceName,
-                         const String& typeName,
+                         const String& typeNameIn,
                          const String& outputDeviceID,
                          const String& inputDeviceID,
                          WASAPIDeviceMode mode)
-        : AudioIODevice (deviceName, typeName),
+        : AudioIODevice (deviceName, typeNameIn),
           Thread ("JUCE WASAPI"),
           outputDeviceId (outputDeviceID),
           inputDeviceId (inputDeviceID),
@@ -1169,7 +1168,7 @@ public:
     {
     }
 
-    ~WASAPIAudioIODevice()
+    ~WASAPIAudioIODevice() override
     {
         cancelPendingUpdate();
         close();
@@ -1341,8 +1340,8 @@ public:
                 return lastError;
             }
 
-            currentBufferSizeSamples = outputDevice != nullptr ? outputDevice->actualBufferSize
-                                                               : inputDevice->actualBufferSize;
+            currentBufferSizeSamples = (int) (outputDevice != nullptr ? outputDevice->actualBufferSize
+                                                                      : inputDevice->actualBufferSize);
         }
 
         if (inputDevice != nullptr)   ResetEvent (inputDevice->clientEvent);
@@ -1442,7 +1441,7 @@ public:
         JUCE_LOAD_WINAPI_FUNCTION (dll, AvSetMmThreadCharacteristicsW, avSetMmThreadCharacteristics, HANDLE, (LPCWSTR, LPDWORD))
         JUCE_LOAD_WINAPI_FUNCTION (dll, AvSetMmThreadPriority, avSetMmThreadPriority, HANDLE, (HANDLE, AVRT_PRIORITY))
 
-        if (avSetMmThreadCharacteristics != 0 && avSetMmThreadPriority != 0)
+        if (avSetMmThreadCharacteristics != nullptr && avSetMmThreadPriority != nullptr)
         {
             DWORD dummy = 0;
 
@@ -1516,8 +1515,12 @@ public:
                 const ScopedTryLock sl (startStopLock);
 
                 if (sl.isLocked() && isStarted)
-                    callback->audioDeviceIOCallback (const_cast<const float**> (inputBuffers), numInputBuffers,
-                                                     outputBuffers, numOutputBuffers, bufferSize);
+                    callback->audioDeviceIOCallbackWithContext (const_cast<const float**> (inputBuffers),
+                                                                numInputBuffers,
+                                                                outputBuffers,
+                                                                numOutputBuffers,
+                                                                bufferSize,
+                                                                {});
                 else
                     outs.clear();
             }
@@ -1666,7 +1669,7 @@ public:
     {
     }
 
-    ~WASAPIAudioIODeviceType()
+    ~WASAPIAudioIODeviceType() override
     {
         if (notifyClient != nullptr)
             enumerator->UnregisterEndpointNotificationCallback (notifyClient);
@@ -1753,13 +1756,13 @@ private:
     {
     public:
         ChangeNotificationClient (WASAPIAudioIODeviceType* d)
-            : ComBaseClassHelper<IMMNotificationClient> (0), device (d) {}
+            : ComBaseClassHelper (0), device (d) {}
 
-        HRESULT STDMETHODCALLTYPE OnDeviceAdded (LPCWSTR)                             { return notify(); }
-        HRESULT STDMETHODCALLTYPE OnDeviceRemoved (LPCWSTR)                           { return notify(); }
-        HRESULT STDMETHODCALLTYPE OnDeviceStateChanged(LPCWSTR, DWORD)                { return notify(); }
-        HRESULT STDMETHODCALLTYPE OnDefaultDeviceChanged (EDataFlow, ERole, LPCWSTR)  { return notify(); }
-        HRESULT STDMETHODCALLTYPE OnPropertyValueChanged (LPCWSTR, const PROPERTYKEY) { return notify(); }
+        JUCE_COMRESULT OnDeviceAdded (LPCWSTR)                             { return notify(); }
+        JUCE_COMRESULT OnDeviceRemoved (LPCWSTR)                           { return notify(); }
+        JUCE_COMRESULT OnDeviceStateChanged(LPCWSTR, DWORD)                { return notify(); }
+        JUCE_COMRESULT OnDefaultDeviceChanged (EDataFlow, ERole, LPCWSTR)  { return notify(); }
+        JUCE_COMRESULT OnPropertyValueChanged (LPCWSTR, const PROPERTYKEY) { return notify(); }
 
     private:
         WeakReference<WASAPIAudioIODeviceType> device;
@@ -1975,4 +1978,6 @@ bool  JUCE_CALLTYPE SystemAudioVolume::setGain (float gain)   { return WasapiCla
 bool  JUCE_CALLTYPE SystemAudioVolume::isMuted()              { return WasapiClasses::MMDeviceMasterVolume().isMuted(); }
 bool  JUCE_CALLTYPE SystemAudioVolume::setMuted (bool mute)   { return WasapiClasses::MMDeviceMasterVolume().setMuted (mute); }
 
+JUCE_END_IGNORE_WARNINGS_GCC_LIKE
+
 } // namespace juce
