@@ -2,15 +2,15 @@
   ==============================================================================
 
    This file is part of the JUCE library.
-   Copyright (c) 2020 - Raw Material Software Limited
+   Copyright (c) 2022 - Raw Material Software Limited
 
    JUCE is an open source library subject to commercial or open-source
    licensing.
 
-   By using JUCE, you agree to the terms of both the JUCE 6 End-User License
-   Agreement and JUCE Privacy Policy (both effective as of the 16th June 2020).
+   By using JUCE, you agree to the terms of both the JUCE 7 End-User License
+   Agreement and JUCE Privacy Policy.
 
-   End User License Agreement: www.juce.com/juce-6-licence
+   End User License Agreement: www.juce.com/juce-7-licence
    Privacy Policy: www.juce.com/juce-privacy-policy
 
    Or: You may also use this code under the terms of the GPL v3 (see
@@ -23,6 +23,8 @@
   ==============================================================================
 */
 
+#ifndef DOXYGEN
+
 // This macro can be set if you need to override this internal name for some reason..
 #ifndef JUCE_STATE_DICTIONARY_KEY
  #define JUCE_STATE_DICTIONARY_KEY   "jucePluginState"
@@ -31,29 +33,24 @@
 namespace juce
 {
 
-#ifndef DOXYGEN
-
 struct AudioUnitHelpers
 {
     class ChannelRemapper
     {
     public:
-        ChannelRemapper (AudioProcessor& p) : processor (p), inputLayoutMap (nullptr), outputLayoutMap (nullptr) {}
-        ~ChannelRemapper() {}
-
-        void alloc()
+        void alloc (AudioProcessor& processor)
         {
-            const int numInputBuses  = AudioUnitHelpers::getBusCount (&processor, true);
-            const int numOutputBuses = AudioUnitHelpers::getBusCount (&processor, false);
+            const int numInputBuses  = AudioUnitHelpers::getBusCount (processor, true);
+            const int numOutputBuses = AudioUnitHelpers::getBusCount (processor, false);
 
-            initializeChannelMapArray (true, numInputBuses);
-            initializeChannelMapArray (false, numOutputBuses);
+            initializeChannelMapArray (processor, true, numInputBuses);
+            initializeChannelMapArray (processor, false, numOutputBuses);
 
             for (int busIdx = 0; busIdx < numInputBuses; ++busIdx)
-                fillLayoutChannelMaps (true, busIdx);
+                fillLayoutChannelMaps (processor, true, busIdx);
 
             for (int busIdx = 0; busIdx < numOutputBuses; ++busIdx)
-                fillLayoutChannelMaps (false, busIdx);
+                fillLayoutChannelMaps (processor, false, busIdx);
         }
 
         void release()
@@ -65,18 +62,17 @@ struct AudioUnitHelpers
             outputLayoutMapStorage.free();
         }
 
-        inline const int* get (bool input, int bus) const noexcept { return (input ? inputLayoutMap : outputLayoutMap) [bus]; }
+        inline const int* get (bool input, int bus) const noexcept { return (input ? inputLayoutMap : outputLayoutMap)[bus]; }
 
     private:
         //==============================================================================
-        AudioProcessor& processor;
         HeapBlock<int*> inputLayoutMapPtrStorage, outputLayoutMapPtrStorage;
         HeapBlock<int>  inputLayoutMapStorage, outputLayoutMapStorage;
-        int** inputLayoutMap;
-        int** outputLayoutMap;
+        int** inputLayoutMap  = nullptr;
+        int** outputLayoutMap = nullptr;
 
         //==============================================================================
-        void initializeChannelMapArray (bool isInput, const int numBuses)
+        void initializeChannelMapArray (AudioProcessor& processor, bool isInput, const int numBuses)
         {
             HeapBlock<int*>& layoutMapPtrStorage = isInput ? inputLayoutMapPtrStorage : outputLayoutMapPtrStorage;
             HeapBlock<int>& layoutMapStorage = isInput ? inputLayoutMapStorage : outputLayoutMapStorage;
@@ -88,7 +84,7 @@ struct AudioUnitHelpers
             layoutMapPtrStorage.calloc (static_cast<size_t> (numBuses));
             layoutMapStorage.calloc (static_cast<size_t> (isInput ? totalInChannels : totalOutChannels));
 
-            layoutMap  = layoutMapPtrStorage. get();
+            layoutMap = layoutMapPtrStorage.get();
 
             int ch = 0;
             for (int busIdx = 0; busIdx < numBuses; ++busIdx)
@@ -98,7 +94,7 @@ struct AudioUnitHelpers
             }
         }
 
-        void fillLayoutChannelMaps (bool isInput, int busNr)
+        void fillLayoutChannelMaps (AudioProcessor& processor, bool isInput, int busNr)
         {
             int* layoutMap = (isInput ? inputLayoutMap : outputLayoutMap)[busNr];
             auto channelFormat = processor.getChannelLayoutOfBus (isInput, busNr);
@@ -119,15 +115,24 @@ struct AudioUnitHelpers
     class CoreAudioBufferList
     {
     public:
-        CoreAudioBufferList() { reset(); }
-
-        //==============================================================================
-        void prepare (int inChannels, int outChannels, int maxFrames)
+        void prepare (const AudioProcessor::BusesLayout& layout, int maxFrames)
         {
-            const int numChannels = jmax (inChannels, outChannels);
+            const auto getChannelOffsets = [] (const auto& range)
+            {
+                std::vector<int> result { 0 };
 
+                for (const auto& bus : range)
+                    result.push_back (result.back() + bus.size());
+
+                return result;
+            };
+
+            inputBusOffsets  = getChannelOffsets (layout.inputBuses);
+            outputBusOffsets = getChannelOffsets (layout.outputBuses);
+
+            const auto numChannels = jmax (inputBusOffsets.back(), outputBusOffsets.back());
             scratch.setSize (numChannels, maxFrames);
-            channels.calloc (static_cast<size_t> (numChannels));
+            channels.resize (static_cast<size_t> (numChannels), nullptr);
 
             reset();
         }
@@ -135,106 +140,112 @@ struct AudioUnitHelpers
         void release()
         {
             scratch.setSize (0, 0);
-            channels.free();
+            channels = {};
+            inputBusOffsets = outputBusOffsets = std::vector<int>();
         }
 
-        void reset() noexcept
+        void reset()
         {
-            pushIdx = 0;
-            popIdx = 0;
-            zeromem (channels.get(), sizeof(float*) * static_cast<size_t> (scratch.getNumChannels()));
+            std::fill (channels.begin(), channels.end(), nullptr);
         }
 
-        //==============================================================================
         float* setBuffer (const int idx, float* ptr = nullptr) noexcept
         {
             jassert (idx < scratch.getNumChannels());
-            return (channels [idx] = uniqueBuffer (idx, ptr));
+            return channels[(size_t) idx] = uniqueBuffer (idx, ptr);
         }
 
-        //==============================================================================
-        float* push() noexcept
-        {
-            jassert (pushIdx < scratch.getNumChannels());
-            return channels [pushIdx++];
-        }
-
-        void push (AudioBufferList& bufferList, const int* channelMap) noexcept
-        {
-            jassert (pushIdx < scratch.getNumChannels());
-
-            if (bufferList.mNumberBuffers > 0)
-            {
-                const UInt32 n = bufferList.mBuffers [0].mDataByteSize /
-                (bufferList.mBuffers [0].mNumberChannels * sizeof (float));
-                const bool isInterleaved = isAudioBufferInterleaved (bufferList);
-                const int numChannels = static_cast<int> (isInterleaved ? bufferList.mBuffers [0].mNumberChannels
-                                                          : bufferList.mNumberBuffers);
-
-                for (int ch = 0; ch < numChannels; ++ch)
-                {
-                    float* data = push();
-
-                    int mappedChannel = channelMap [ch];
-                    if (isInterleaved || static_cast<float*> (bufferList.mBuffers [mappedChannel].mData) != data)
-                        copyAudioBuffer (bufferList, mappedChannel, n, data);
-                }
-            }
-        }
-
-        //==============================================================================
-        float* pop() noexcept
-        {
-            jassert (popIdx < scratch.getNumChannels());
-            return channels[popIdx++];
-        }
-
-        void pop (AudioBufferList& buffer, const int* channelMap) noexcept
-        {
-            if (buffer.mNumberBuffers > 0)
-            {
-                const UInt32 n = buffer.mBuffers [0].mDataByteSize / (buffer.mBuffers [0].mNumberChannels * sizeof (float));
-                const bool isInterleaved = isAudioBufferInterleaved (buffer);
-                const int numChannels = static_cast<int> (isInterleaved ? buffer.mBuffers [0].mNumberChannels : buffer.mNumberBuffers);
-
-                for (int ch = 0; ch < numChannels; ++ch)
-                {
-                    int mappedChannel = channelMap [ch];
-                    float* nextBuffer = pop();
-
-                    if (nextBuffer == buffer.mBuffers [mappedChannel].mData && ! isInterleaved)
-                        continue; // no copying necessary
-
-                    if (buffer.mBuffers [mappedChannel].mData == nullptr && ! isInterleaved)
-                        buffer.mBuffers [mappedChannel].mData = nextBuffer;
-                    else
-                        copyAudioBuffer (nextBuffer, mappedChannel, n, buffer);
-                }
-            }
-        }
-
-        //==============================================================================
         AudioBuffer<float>& getBuffer (UInt32 frames) noexcept
         {
-            jassert (pushIdx == scratch.getNumChannels());
+            jassert (std::none_of (channels.begin(), channels.end(), [] (auto* x) { return x == nullptr; }));
 
-          #if JUCE_DEBUG
-            for (int i = 0; i < pushIdx; ++i)
-                jassert (channels [i] != nullptr);
-          #endif
+            const auto channelPtr = channels.empty() ? scratch.getArrayOfWritePointers() : channels.data();
+            mutableBuffer.setDataToReferTo (channelPtr, (int) channels.size(), static_cast<int> (frames));
 
-            mutableBuffer.setDataToReferTo (channels, pushIdx, static_cast<int> (frames));
             return mutableBuffer;
         }
 
+        void set (int bus, AudioBufferList& bufferList, const int* channelMap) noexcept
+        {
+            if (bufferList.mNumberBuffers <= 0 || ! isPositiveAndBelow (bus, inputBusOffsets.size() - 1))
+                return;
+
+            const auto n = (UInt32) (bufferList.mBuffers[0].mDataByteSize / (bufferList.mBuffers[0].mNumberChannels * sizeof (float)));
+            const auto isInterleaved = isAudioBufferInterleaved (bufferList);
+            const auto numChannels = (int) (isInterleaved ? bufferList.mBuffers[0].mNumberChannels
+                                                          : bufferList.mNumberBuffers);
+
+            for (int ch = 0; ch < numChannels; ++ch)
+            {
+                float* data = channels[(size_t) (inputBusOffsets[(size_t) bus] + ch)];
+
+                const auto mappedChannel = channelMap[ch];
+
+                if (isInterleaved || static_cast<float*> (bufferList.mBuffers[mappedChannel].mData) != data)
+                    copyAudioBuffer (bufferList, mappedChannel, n, data);
+            }
+        }
+
+        void get (int bus, AudioBufferList& buffer, const int* channelMap) noexcept
+        {
+            if (buffer.mNumberBuffers <= 0 || ! isPositiveAndBelow (bus, outputBusOffsets.size() - 1))
+                return;
+
+            const auto n = (UInt32) (buffer.mBuffers[0].mDataByteSize / (buffer.mBuffers[0].mNumberChannels * sizeof (float)));
+            const auto isInterleaved = isAudioBufferInterleaved (buffer);
+            const auto numChannels = (int) (isInterleaved ? buffer.mBuffers[0].mNumberChannels
+                                                          : buffer.mNumberBuffers);
+
+            for (int ch = 0; ch < numChannels; ++ch)
+            {
+                float* data = channels[(size_t) (outputBusOffsets[(size_t) bus] + ch)];
+
+                const auto mappedChannel = channelMap[ch];
+
+                if (data == buffer.mBuffers[mappedChannel].mData && ! isInterleaved)
+                    continue; // no copying necessary
+
+                if (buffer.mBuffers[mappedChannel].mData == nullptr && ! isInterleaved)
+                    buffer.mBuffers[mappedChannel].mData = data;
+                else
+                    copyAudioBuffer (data, mappedChannel, n, buffer);
+            }
+        }
+
+        void clearInputBus (int index, int bufferLength)
+        {
+            if (isPositiveAndBelow (index, inputBusOffsets.size() - 1))
+                clearChannels ({ inputBusOffsets[(size_t) index], inputBusOffsets[(size_t) (index + 1)] }, bufferLength);
+        }
+
+        void clearUnusedChannels (int bufferLength)
+        {
+            jassert (! inputBusOffsets .empty());
+            jassert (! outputBusOffsets.empty());
+
+            clearChannels ({ inputBusOffsets.back(), outputBusOffsets.back() }, bufferLength);
+        }
+
     private:
+        void clearChannels (Range<int> range, int bufferLength)
+        {
+            jassert (bufferLength <= scratch.getNumSamples());
+
+            if (range.getEnd() <= (int) channels.size())
+            {
+                std::for_each (channels.begin() + range.getStart(),
+                               channels.begin() + range.getEnd(),
+                               [bufferLength] (float* ptr) { zeromem (ptr, sizeof (float) * (size_t) bufferLength); });
+            }
+        }
+
         float* uniqueBuffer (int idx, float* buffer) noexcept
         {
             if (buffer == nullptr)
                 return scratch.getWritePointer (idx);
 
             for (int ch = 0; ch < idx; ++ch)
-                if (buffer == channels[ch])
+                if (buffer == channels[(size_t) ch])
                     return scratch.getWritePointer (idx);
 
             return buffer;
@@ -242,8 +253,8 @@ struct AudioUnitHelpers
 
         //==============================================================================
         AudioBuffer<float> scratch, mutableBuffer;
-        HeapBlock<float*> channels;
-        int pushIdx, popIdx;
+        std::vector<float*> channels;
+        std::vector<int> inputBusOffsets, outputBusOffsets;
     };
 
     static bool isAudioBufferInterleaved (const AudioBufferList& audioBuffer) noexcept
@@ -307,7 +318,7 @@ struct AudioUnitHelpers
     static bool isLayoutSupported (const AudioProcessor& processor,
                                    bool isInput, int busIdx,
                                    int numChannels,
-                                   const short (&channelLayoutList) [numLayouts][2],
+                                   const short (&channelLayoutList)[numLayouts][2],
                                    bool hasLayoutMap = true)
     {
         if (const AudioProcessor::Bus* bus = processor.getBus (isInput, busIdx))
@@ -334,8 +345,8 @@ struct AudioUnitHelpers
     {
         Array<AUChannelInfo> channelInfo;
 
-        auto hasMainInputBus  = (AudioUnitHelpers::getBusCount (&processor, true)  > 0);
-        auto hasMainOutputBus = (AudioUnitHelpers::getBusCount (&processor, false) > 0);
+        auto hasMainInputBus  = (AudioUnitHelpers::getBusCountForWrapper (processor, true)  > 0);
+        auto hasMainOutputBus = (AudioUnitHelpers::getBusCountForWrapper (processor, false) > 0);
 
         if ((! hasMainInputBus) && (! hasMainOutputBus))
         {
@@ -353,11 +364,22 @@ struct AudioUnitHelpers
         auto defaultInputs  = processor.getChannelCountOfBus (true,  0);
         auto defaultOutputs = processor.getChannelCountOfBus (false, 0);
 
-        SortedSet<int> supportedChannels;
+        struct Channels
+        {
+            SInt16 ins, outs;
+
+            std::pair<SInt16, SInt16> makePair() const noexcept { return std::make_pair (ins, outs); }
+
+            bool operator<  (const Channels& other) const noexcept { return makePair() <  other.makePair(); }
+            bool operator== (const Channels& other) const noexcept { return makePair() == other.makePair(); }
+        };
+
+        SortedSet<Channels> supportedChannels;
 
         // add the current configuration
         if (defaultInputs != 0 || defaultOutputs != 0)
-            supportedChannels.add ((defaultInputs << 16) | defaultOutputs);
+            supportedChannels.add ({ static_cast<SInt16> (defaultInputs),
+                                     static_cast<SInt16> (defaultOutputs) });
 
         for (auto inChanNum = hasMainInputBus ? 1 : 0; inChanNum <= (hasMainInputBus ? maxNumChanToCheckFor : 0); ++inChanNum)
         {
@@ -375,19 +397,16 @@ struct AudioUnitHelpers
                     if (! isNumberOfChannelsSupported (outBus, outChanNum, outLayout))
                         continue;
 
-                supportedChannels.add (((hasMainInputBus  ? outLayout.getMainInputChannels()  : 0) << 16)
-                                      | (hasMainOutputBus ? outLayout.getMainOutputChannels() : 0));
+                supportedChannels.add ({ static_cast<SInt16> (hasMainInputBus  ? outLayout.getMainInputChannels()  : 0),
+                                         static_cast<SInt16> (hasMainOutputBus ? outLayout.getMainOutputChannels() : 0) });
             }
         }
 
         auto hasInOutMismatch = false;
 
-        for (auto supported : supportedChannels)
+        for (const auto& supported : supportedChannels)
         {
-            auto numInputs  = (supported >> 16) & 0xffff;
-            auto numOutputs = (supported >> 0)  & 0xffff;
-
-            if (numInputs != numOutputs)
+            if (supported.ins != supported.outs)
             {
                 hasInOutMismatch = true;
                 break;
@@ -398,7 +417,8 @@ struct AudioUnitHelpers
 
         for (auto inChanNum = hasMainInputBus ? 1 : 0; inChanNum <= (hasMainInputBus ? maxNumChanToCheckFor : 0); ++inChanNum)
         {
-            auto channelConfiguration = (inChanNum << 16) | (hasInOutMismatch ? defaultOutputs : inChanNum);
+            Channels channelConfiguration { static_cast<SInt16> (inChanNum),
+                                            static_cast<SInt16> (hasInOutMismatch ? defaultOutputs : inChanNum) };
 
             if (! supportedChannels.contains (channelConfiguration))
             {
@@ -409,7 +429,8 @@ struct AudioUnitHelpers
 
         for (auto outChanNum = hasMainOutputBus ? 1 : 0; outChanNum <= (hasMainOutputBus ? maxNumChanToCheckFor : 0); ++outChanNum)
         {
-            auto channelConfiguration = ((hasInOutMismatch ? defaultInputs : outChanNum) << 16) | outChanNum;
+            Channels channelConfiguration { static_cast<SInt16> (hasInOutMismatch ? defaultInputs : outChanNum),
+                                            static_cast<SInt16> (outChanNum) };
 
             if (! supportedChannels.contains (channelConfiguration))
             {
@@ -418,16 +439,13 @@ struct AudioUnitHelpers
             }
         }
 
-        for (auto supported : supportedChannels)
+        for (const auto& supported : supportedChannels)
         {
-            auto numInputs  = (supported >> 16) & 0xffff;
-            auto numOutputs = (supported >> 0)  & 0xffff;
-
             AUChannelInfo info;
 
             // see here: https://developer.apple.com/library/mac/documentation/MusicAudio/Conceptual/AudioUnitProgrammingGuide/TheAudioUnit/TheAudioUnit.html
-            info.inChannels  = static_cast<SInt16> (hasMainInputBus  ? (hasUnsupportedInput  ? numInputs :  (hasInOutMismatch && (! hasUnsupportedOutput) ? -2 : -1)) : 0);
-            info.outChannels = static_cast<SInt16> (hasMainOutputBus ? (hasUnsupportedOutput ? numOutputs : (hasInOutMismatch && (! hasUnsupportedInput)  ? -2 : -1)) : 0);
+            info.inChannels  = static_cast<SInt16> (hasMainInputBus  ? (hasUnsupportedInput  ? supported.ins  : (hasInOutMismatch && (! hasUnsupportedOutput) ? -2 : -1)) : 0);
+            info.outChannels = static_cast<SInt16> (hasMainOutputBus ? (hasUnsupportedOutput ? supported.outs : (hasInOutMismatch && (! hasUnsupportedInput)  ? -2 : -1)) : 0);
 
             if (info.inChannels == -2 && info.outChannels == -2)
                 info.inChannels = -1;
@@ -464,9 +482,9 @@ struct AudioUnitHelpers
     }
 
     //==============================================================================
-    static int getBusCount (const AudioProcessor* juceFilter, bool isInput)
+    static int getBusCount (const AudioProcessor& juceFilter, bool isInput)
     {
-        int busCount = juceFilter->getBusCount (isInput);
+        int busCount = juceFilter.getBusCount (isInput);
 
        #ifdef JucePlugin_PreferredChannelConfigurations
         short configs[][2] = {JucePlugin_PreferredChannelConfigurations};
@@ -484,6 +502,17 @@ struct AudioUnitHelpers
         return busCount;
     }
 
+    static int getBusCountForWrapper (const AudioProcessor& juceFilter, bool isInput)
+    {
+       #if JucePlugin_IsMidiEffect
+        const auto numRequiredBuses = isInput ? 0 : 1;
+       #else
+        const auto numRequiredBuses = 0;
+       #endif
+
+        return jmax (numRequiredBuses, getBusCount (juceFilter, isInput));
+    }
+
     static bool setBusesLayout (AudioProcessor* juceFilter, const AudioProcessor::BusesLayout& requestedLayouts)
     {
        #ifdef JucePlugin_PreferredChannelConfigurations
@@ -494,7 +523,7 @@ struct AudioUnitHelpers
             const bool isInput = (dir == 0);
 
             const int actualBuses = juceFilter->getBusCount (isInput);
-            const int auNumBuses  = getBusCount (juceFilter, isInput);
+            const int auNumBuses  = getBusCount (*juceFilter, isInput);
             Array<AudioChannelSet>& buses = (isInput ? copy.inputBuses : copy.outputBuses);
 
             for (int i = auNumBuses; i < actualBuses; ++i)
@@ -517,7 +546,7 @@ struct AudioUnitHelpers
             const bool isInput = (dir == 0);
 
             const int actualBuses = juceFilter->getBusCount (isInput);
-            const int auNumBuses  = getBusCount (juceFilter, isInput);
+            const int auNumBuses  = getBusCount (*juceFilter, isInput);
             auto& buses = (isInput ? layout.inputBuses : layout.outputBuses);
 
             for (int i = auNumBuses; i < actualBuses; ++i)
@@ -531,6 +560,6 @@ struct AudioUnitHelpers
     }
 };
 
-#endif // ! DOXYGEN
-
 } // namespace juce
+
+#endif
