@@ -2,15 +2,15 @@
   ==============================================================================
 
    This file is part of the JUCE library.
-   Copyright (c) 2020 - Raw Material Software Limited
+   Copyright (c) 2022 - Raw Material Software Limited
 
    JUCE is an open source library subject to commercial or open-source
    licensing.
 
-   By using JUCE, you agree to the terms of both the JUCE 6 End-User License
-   Agreement and JUCE Privacy Policy (both effective as of the 16th June 2020).
+   By using JUCE, you agree to the terms of both the JUCE 7 End-User License
+   Agreement and JUCE Privacy Policy.
 
-   End User License Agreement: www.juce.com/juce-6-licence
+   End User License Agreement: www.juce.com/juce-7-licence
    Privacy Policy: www.juce.com/juce-privacy-policy
 
    Or: You may also use this code under the terms of the GPL v3 (see
@@ -26,7 +26,7 @@
 namespace juce
 {
 
-#if (JUCE_PLUGINHOST_VST3 && (JUCE_MAC || JUCE_WINDOWS || JUCE_LINUX)) || DOXYGEN
+#if (JUCE_PLUGINHOST_VST3 && (JUCE_MAC || JUCE_WINDOWS || JUCE_LINUX || JUCE_BSD)) || DOXYGEN
 
 /**
     Implements a plugin format for VST3s.
@@ -43,11 +43,17 @@ public:
     ~VST3PluginFormat() override;
 
     //==============================================================================
+   #ifndef DOXYGEN
     /** Attempts to reload a VST3 plugin's state from some preset file data.
 
         @see VSTPluginFormat::loadFromFXBFile
     */
+    [[deprecated ("Instead of using this function, use AudioPluginInstance::getExtensions() "
+                 "to visit the ExtensionsVisitor::VST3 struct for the instance, if it exists. "
+                 "Then, call ExtensionsVisitor::VST3::setPreset() to set the state using the "
+                 "contents of a vstpreset file.")]]
     static bool setStateFromVSTPresetFile (AudioPluginInstance*, const MemoryBlock&);
+   #endif
 
     //==============================================================================
     static String getFormatName()                   { return "VST3"; }
@@ -62,6 +68,7 @@ public:
     StringArray searchPathsForPlugins (const FileSearchPath&, bool recursive, bool) override;
     bool doesPluginStillExist (const PluginDescription&) override;
     FileSearchPath getDefaultLocationsToSearch() override;
+    void createARAFactoryAsync (const PluginDescription&, ARAFactoryCreationCallback callback) override;
 
 private:
     //==============================================================================
