@@ -2,15 +2,15 @@
   ==============================================================================
 
    This file is part of the JUCE library.
-   Copyright (c) 2020 - Raw Material Software Limited
+   Copyright (c) 2022 - Raw Material Software Limited
 
    JUCE is an open source library subject to commercial or open-source
    licensing.
 
-   By using JUCE, you agree to the terms of both the JUCE 6 End-User License
-   Agreement and JUCE Privacy Policy (both effective as of the 16th June 2020).
+   By using JUCE, you agree to the terms of both the JUCE 7 End-User License
+   Agreement and JUCE Privacy Policy.
 
-   End User License Agreement: www.juce.com/juce-6-licence
+   End User License Agreement: www.juce.com/juce-7-licence
    Privacy Policy: www.juce.com/juce-privacy-policy
 
    Or: You may also use this code under the terms of the GPL v3 (see
@@ -27,40 +27,26 @@ namespace juce
 {
 
 //==============================================================================
-AudioProcessorValueTreeState::Parameter::Parameter (const String& parameterID,
+
+AudioProcessorValueTreeState::Parameter::Parameter (const ParameterID& parameterID,
                                                     const String& parameterName,
-                                                    const String& labelText,
                                                     NormalisableRange<float> valueRange,
                                                     float defaultParameterValue,
-                                                    std::function<String (float)> valueToTextFunction,
-                                                    std::function<float (const String&)> textToValueFunction,
-                                                    bool isMetaParameter,
-                                                    bool isAutomatableParameter,
-                                                    bool isDiscrete,
-                                                    AudioProcessorParameter::Category parameterCategory,
-                                                    bool isBoolean)
+                                                    const AudioProcessorValueTreeStateParameterAttributes& attributes)
     : AudioParameterFloat (parameterID,
                            parameterName,
                            valueRange,
                            defaultParameterValue,
-                           labelText,
-                           parameterCategory,
-                           valueToTextFunction == nullptr ? std::function<String (float v, int)>()
-                                                          : [valueToTextFunction] (float v, int) { return valueToTextFunction (v); },
-                           std::move (textToValueFunction)),
+                           attributes.getAudioParameterFloatAttributes()),
       unsnappedDefault (valueRange.convertTo0to1 (defaultParameterValue)),
-      metaParameter (isMetaParameter),
-      automatable (isAutomatableParameter),
-      discrete (isDiscrete),
-      boolean (isBoolean)
+      discrete (attributes.getDiscrete()),
+      boolean (attributes.getBoolean())
 {
 }
 
 float AudioProcessorValueTreeState::Parameter::getDefaultValue() const  { return unsnappedDefault; }
 int AudioProcessorValueTreeState::Parameter::getNumSteps() const        { return RangedAudioParameter::getNumSteps(); }
 
-bool AudioProcessorValueTreeState::Parameter::isMetaParameter() const   { return metaParameter; }
-bool AudioProcessorValueTreeState::Parameter::isAutomatable() const     { return automatable; }
 bool AudioProcessorValueTreeState::Parameter::isDiscrete() const        { return discrete; }
 bool AudioProcessorValueTreeState::Parameter::isBoolean() const         { return boolean; }
 
@@ -162,7 +148,7 @@ private:
             return;
 
         unnormalisedValue = newValue;
-        listeners.call ([=] (Listener& l) { l.parameterChanged (parameter.paramID, unnormalisedValue); });
+        listeners.call ([this] (Listener& l) { l.parameterChanged (parameter.paramID, unnormalisedValue); });
         listenersNeedCalling = false;
         needsUpdate = true;
     }
@@ -303,18 +289,21 @@ RangedAudioParameter* AudioProcessorValueTreeState::createAndAddParameter (const
                                                                            AudioProcessorParameter::Category category,
                                                                            bool isBooleanParameter)
 {
+    auto attributes = AudioProcessorValueTreeStateParameterAttributes()
+                          .withLabel (labelText)
+                          .withStringFromValueFunction ([fn = std::move (valueToTextFunction)] (float v, int) { return fn (v); })
+                          .withValueFromStringFunction (std::move (textToValueFunction))
+                          .withMeta (isMetaParameter)
+                          .withAutomatable (isAutomatableParameter)
+                          .withDiscrete (isDiscreteParameter)
+                          .withCategory (category)
+                          .withBoolean (isBooleanParameter);
+
     return createAndAddParameter (std::make_unique<Parameter> (paramID,
                                                                paramName,
-                                                               labelText,
                                                                range,
                                                                defaultVal,
-                                                               std::move (valueToTextFunction),
-                                                               std::move (textToValueFunction),
-                                                               isMetaParameter,
-                                                               isAutomatableParameter,
-                                                               isDiscreteParameter,
-                                                               category,
-                                                               isBooleanParameter));
+                                                               std::move (attributes)));
 }
 
 RangedAudioParameter* AudioProcessorValueTreeState::createAndAddParameter (std::unique_ptr<RangedAudioParameter> param)
@@ -484,6 +473,7 @@ void AudioProcessorValueTreeState::timerCallback()
 }
 
 //==============================================================================
+#if ! JUCE_AUDIOPROCESSOR_NO_GUI
 template <typename Attachment, typename Control>
 std::unique_ptr<Attachment> makeAttachment (const AudioProcessorValueTreeState& stateToUse,
                                             const String& parameterID,
@@ -516,6 +506,7 @@ AudioProcessorValueTreeState::ButtonAttachment::ButtonAttachment (AudioProcessor
     : attachment (makeAttachment<ButtonParameterAttachment> (stateToUse, parameterID, button))
 {
 }
+#endif
 
 //==============================================================================
 //==============================================================================
@@ -533,7 +524,7 @@ struct ParameterAdapterTests  : public UnitTest
         {
             const auto test = [&] (NormalisableRange<float> range, float value)
             {
-                AudioParameterFloat param ({}, {}, range, value, {});
+                AudioParameterFloat param ({}, {}, range, value);
 
                 AudioProcessorValueTreeState::ParameterAdapter adapter (param);
 
@@ -548,7 +539,7 @@ struct ParameterAdapterTests  : public UnitTest
         {
             const auto test = [&] (NormalisableRange<float> range, float value)
             {
-                AudioParameterFloat param ({}, {}, range, {}, {});
+                AudioParameterFloat param ({}, {}, range, {});
                 AudioProcessorValueTreeState::ParameterAdapter adapter (param);
 
                 adapter.setDenormalisedValue (value);
@@ -565,7 +556,7 @@ struct ParameterAdapterTests  : public UnitTest
         {
             const auto test = [&] (NormalisableRange<float> range, float value, String expected)
             {
-                AudioParameterFloat param ({}, {}, range, {}, {});
+                AudioParameterFloat param ({}, {}, range, {});
                 AudioProcessorValueTreeState::ParameterAdapter adapter (param);
 
                 expectEquals (adapter.getTextForDenormalisedValue (value), expected);
@@ -581,7 +572,7 @@ struct ParameterAdapterTests  : public UnitTest
         {
             const auto test = [&] (NormalisableRange<float> range, String text, float expected)
             {
-                AudioParameterFloat param ({}, {}, range, {}, {});
+                AudioParameterFloat param ({}, {}, range, {});
                 AudioProcessorValueTreeState::ParameterAdapter adapter (param);
 
                 expectEquals (adapter.getDenormalisedValueForText (text), expected);
@@ -621,6 +612,7 @@ private:
     using Parameter = AudioProcessorValueTreeState::Parameter;
     using ParameterGroup = AudioProcessorParameterGroup;
     using ParameterLayout = AudioProcessorValueTreeState::ParameterLayout;
+    using Attributes = AudioProcessorValueTreeStateParameterAttributes;
 
     class TestAudioProcessor : public AudioProcessor
     {
@@ -668,6 +660,7 @@ public:
         : UnitTest ("Audio Processor Value Tree State", UnitTestCategories::audioProcessorParameters)
     {}
 
+    JUCE_BEGIN_IGNORE_WARNINGS_MSVC (6262)
     void runTest() override
     {
         ScopedJuceInitialiser_GUI scopedJuceInitialiser_gui;
@@ -676,8 +669,11 @@ public:
         {
             TestAudioProcessor proc;
 
-            proc.state.createAndAddParameter (std::make_unique<Parameter> (String(), String(), String(), NormalisableRange<float>(),
-                                                                           0.0f, nullptr, nullptr));
+            proc.state.createAndAddParameter (std::make_unique<Parameter> (
+                String(),
+                String(),
+                NormalisableRange<float>(),
+                0.0f));
 
             expectEquals (proc.getParameters().size(), 1);
         }
@@ -687,8 +683,11 @@ public:
             TestAudioProcessor proc;
 
             const auto key = "id";
-            const auto param = proc.state.createAndAddParameter (std::make_unique<Parameter> (key, String(), String(), NormalisableRange<float>(),
-                                                                                              0.0f, nullptr, nullptr));
+            const auto param = proc.state.createAndAddParameter (std::make_unique<Parameter> (
+                                   key,
+                                   String(),
+                                   NormalisableRange<float>(),
+                                   0.0f));
 
             expect (proc.state.getParameter (key) == param);
         }
@@ -696,10 +695,10 @@ public:
         beginTest ("After construction, the value tree has the expected format");
         {
             TestAudioProcessor proc ({
-                std::make_unique<AudioProcessorParameterGroup> ("", "", "",
+                std::make_unique<AudioProcessorParameterGroup> ("A", "", "",
                     std::make_unique<AudioParameterBool> ("a", "", false),
                     std::make_unique<AudioParameterFloat> ("b", "", NormalisableRange<float>{}, 0.0f)),
-                std::make_unique<AudioProcessorParameterGroup> ("", "", "",
+                std::make_unique<AudioProcessorParameterGroup> ("B", "", "",
                     std::make_unique<AudioParameterInt> ("c", "", 0, 1, 0),
                     std::make_unique<AudioParameterChoice> ("d", "", StringArray { "foo", "bar" }, 0)) });
 
@@ -720,8 +719,12 @@ public:
             TestAudioProcessor proc;
 
             const auto key = "id";
-            const auto param = proc.state.createAndAddParameter (std::make_unique<Parameter> (key, String(), String(), NormalisableRange<float>(),
-                                                                                              0.0f, nullptr, nullptr, true));
+            const auto param = proc.state.createAndAddParameter (std::make_unique<Parameter> (
+                                   key,
+                                   String(),
+                                   NormalisableRange<float>(),
+                                   0.0f,
+                                   Attributes().withMeta (true)));
 
             expect (param->isMetaParameter());
         }
@@ -731,8 +734,12 @@ public:
             TestAudioProcessor proc;
 
             const auto key = "id";
-            const auto param = proc.state.createAndAddParameter (std::make_unique<Parameter> (key, String(), String(), NormalisableRange<float>(),
-                                                                                              0.0f, nullptr, nullptr, false, true));
+            const auto param = proc.state.createAndAddParameter (std::make_unique<Parameter> (
+                                   key,
+                                   String(),
+                                   NormalisableRange<float>(),
+                                   0.0f,
+                                   Attributes().withAutomatable (true)));
 
             expect (param->isAutomatable());
         }
@@ -742,8 +749,12 @@ public:
             TestAudioProcessor proc;
 
             const auto key = "id";
-            const auto param = proc.state.createAndAddParameter (std::make_unique<Parameter> (key, String(), String(), NormalisableRange<float>(),
-                                                                                              0.0f, nullptr, nullptr, false, false, true));
+            const auto param = proc.state.createAndAddParameter (std::make_unique<Parameter> (
+                                   key,
+                                   String(),
+                                   NormalisableRange<float>(),
+                                   0.0f,
+                                   Attributes().withDiscrete (true)));
 
             expect (param->isDiscrete());
         }
@@ -753,9 +764,12 @@ public:
             TestAudioProcessor proc;
 
             const auto key = "id";
-            const auto param = proc.state.createAndAddParameter (std::make_unique<Parameter> (key, String(), String(), NormalisableRange<float>(),
-                                                                                              0.0f, nullptr, nullptr, false, false, false,
-                                                                                              AudioProcessorParameter::Category::inputMeter));
+            const auto param = proc.state.createAndAddParameter (std::make_unique<Parameter> (
+                                   key,
+                                   String(),
+                                   NormalisableRange<float>(),
+                                   0.0f,
+                                   Attributes().withCategory (AudioProcessorParameter::Category::inputMeter)));
 
             expect (param->category == AudioProcessorParameter::Category::inputMeter);
         }
@@ -765,9 +779,12 @@ public:
             TestAudioProcessor proc;
 
             const auto key = "id";
-            const auto param = proc.state.createAndAddParameter (std::make_unique<Parameter> (key, String(), String(), NormalisableRange<float>(),
-                                                                                              0.0f, nullptr, nullptr, false, false, false,
-                                                                                              AudioProcessorParameter::Category::genericParameter, true));
+            const auto param = proc.state.createAndAddParameter (std::make_unique<Parameter> (
+                                   key,
+                                   String(),
+                                   NormalisableRange<float>(),
+                                   0.0f,
+                                   Attributes().withBoolean (true)));
 
             expect (param->isBoolean());
         }
@@ -787,11 +804,17 @@ public:
         {
             TestAudioProcessor proc;
             const auto key = "id";
-            const auto param = proc.state.createAndAddParameter (std::make_unique<Parameter> (key, String(), String(), NormalisableRange<float>(),
-                                                                                              0.0f, nullptr, nullptr));
+            const auto param = proc.state.createAndAddParameter (std::make_unique<Parameter> (
+                                   key,
+                                   String(),
+                                   NormalisableRange<float>(),
+                                   0.0f));
 
-            proc.state.createAndAddParameter (std::make_unique<Parameter> (key, String(), String(), NormalisableRange<float>(),
-                                                                           0.0f, nullptr, nullptr));
+            proc.state.createAndAddParameter (std::make_unique<Parameter> (
+                key,
+                String(),
+                NormalisableRange<float>(),
+                0.0f));
 
             expectEquals (proc.getParameters().size(), 1);
             expect (proc.getParameters().getFirst() == param);
@@ -801,8 +824,11 @@ public:
         {
             TestAudioProcessor proc;
             const auto key = "id";
-            const auto param = proc.state.createAndAddParameter (std::make_unique<Parameter> (key, String(), String(), NormalisableRange<float>(),
-                                                                                              0.0f, nullptr, nullptr));
+            const auto param = proc.state.createAndAddParameter (std::make_unique<Parameter> (
+                                   key,
+                                   String(),
+                                   NormalisableRange<float>(),
+                                   0.0f));
 
             const auto value = 0.5f;
             param->setValueNotifyingHost (value);
@@ -819,11 +845,8 @@ public:
             proc.state.createAndAddParameter (std::make_unique<Parameter> (
                 key,
                 String(),
-                String(),
                 NormalisableRange<float> (0.0f, 100.0f, 10.0f),
-                value,
-                nullptr,
-                nullptr));
+                value));
 
             expectEquals (proc.state.getRawParameterValue (key)->load(), value);
         }
@@ -833,8 +856,11 @@ public:
             Listener listener;
             TestAudioProcessor proc;
             const auto key = "id";
-            const auto param = proc.state.createAndAddParameter (std::make_unique<Parameter> (key, String(), String(), NormalisableRange<float>(),
-                                                                                              0.0f, nullptr, nullptr));
+            const auto param = proc.state.createAndAddParameter (std::make_unique<Parameter> (
+                                   key,
+                                   String(),
+                                   NormalisableRange<float>(),
+                                   0.0f));
             proc.state.addParameterListener (key, &listener);
 
             const auto value = 0.5f;
@@ -890,8 +916,11 @@ public:
             TestAudioProcessor proc;
             const auto key = "id";
             const auto initialValue = 0.2f;
-            auto param = proc.state.createAndAddParameter (std::make_unique<Parameter> (key, String(), String(), NormalisableRange<float>(),
-                                                                                        initialValue, nullptr, nullptr));
+            auto param = proc.state.createAndAddParameter (std::make_unique<Parameter> (
+                             key,
+                             String(),
+                             NormalisableRange<float>(),
+                             initialValue));
             proc.state.state = ValueTree { "state" };
 
             auto value = proc.state.getParameterAsValue (key);
@@ -925,8 +954,11 @@ public:
             Listener listener;
             TestAudioProcessor proc;
             const auto key = "id";
-            proc.state.createAndAddParameter (std::make_unique<Parameter> (key, String(), String(), NormalisableRange<float>(),
-                                                                           0.0f, nullptr, nullptr));
+            proc.state.createAndAddParameter (std::make_unique<Parameter> (
+                key,
+                String(),
+                NormalisableRange<float>(),
+                0.0f));
             proc.state.addParameterListener (key, &listener);
             proc.state.state = ValueTree { "state" };
 
@@ -952,6 +984,7 @@ public:
             expectEquals (listener.id, String (key));
         }
     }
+    JUCE_END_IGNORE_WARNINGS_MSVC
 };
 
 static AudioProcessorValueTreeStateTests audioProcessorValueTreeStateTests;
