@@ -2,7 +2,7 @@
   ==============================================================================
 
    This file is part of the JUCE library.
-   Copyright (c) 2017 - ROLI Ltd.
+   Copyright (c) 2022 - Raw Material Software Limited
 
    JUCE is an open source library subject to commercial or open-source
    licensing.
@@ -63,6 +63,8 @@ class UnitTestRunner;
     To run a test, use the UnitTestRunner class.
 
     @see UnitTestRunner
+
+    @tags{Core}
 */
 class JUCE_API  UnitTest
 {
@@ -298,9 +300,8 @@ private:
     }
 
     //==============================================================================
-    const String name;
-    const String category;
-    UnitTestRunner* runner;
+    const String name, category;
+    UnitTestRunner* runner = nullptr;
 
     JUCE_DECLARE_NON_COPYABLE (UnitTest)
 };
@@ -317,6 +318,8 @@ private:
     perform custom behaviour when each test completes.
 
     @see UnitTest
+
+    @tags{Core}
 */
 class JUCE_API  UnitTestRunner
 {
@@ -373,18 +376,31 @@ public:
     */
     struct TestResult
     {
+        TestResult() = default;
+
+        explicit TestResult (const String& name, const String& subCategory)
+             : unitTestName (name),
+               subcategoryName (subCategory)
+        {
+        }
+
         /** The main name of this test (i.e. the name of the UnitTest object being run). */
         String unitTestName;
         /** The name of the current subcategory (i.e. the name that was set when UnitTest::beginTest() was called). */
         String subcategoryName;
 
         /** The number of UnitTest::expect() calls that succeeded. */
-        int passes;
+        int passes = 0;
         /** The number of UnitTest::expect() calls that failed. */
-        int failures;
+        int failures = 0;
 
         /** A list of messages describing the failed tests. */
         StringArray messages;
+
+        /** The time at which this test was started. */
+        Time startTime = Time::getCurrentTime();
+        /** The time at which this test ended. */
+        Time endTime;
     };
 
     /** Returns the number of TestResult objects that have been performed.
@@ -418,10 +434,10 @@ private:
     //==============================================================================
     friend class UnitTest;
 
-    UnitTest* currentTest;
+    UnitTest* currentTest = nullptr;
     String currentSubCategory;
-    OwnedArray <TestResult, CriticalSection> results;
-    bool assertOnFailure, logPasses;
+    OwnedArray<TestResult, CriticalSection> results;
+    bool assertOnFailure = true, logPasses = false;
     Random randomForTest;
 
     void beginNewTest (UnitTest* test, const String& subCategory);
