@@ -2,15 +2,15 @@
   ==============================================================================
 
    This file is part of the JUCE library.
-   Copyright (c) 2020 - Raw Material Software Limited
+   Copyright (c) 2022 - Raw Material Software Limited
 
    JUCE is an open source library subject to commercial or open-source
    licensing.
 
-   By using JUCE, you agree to the terms of both the JUCE 6 End-User License
-   Agreement and JUCE Privacy Policy (both effective as of the 16th June 2020).
+   By using JUCE, you agree to the terms of both the JUCE 7 End-User License
+   Agreement and JUCE Privacy Policy.
 
-   End User License Agreement: www.juce.com/juce-6-licence
+   End User License Agreement: www.juce.com/juce-7-licence
    Privacy Policy: www.juce.com/juce-privacy-policy
 
    Or: You may also use this code under the terms of the GPL v3 (see
@@ -299,35 +299,35 @@ public:
 
     //==============================================================================
     /** Returns a copy of this colour with a different hue. */
-    Colour withHue (float newHue) const noexcept;
+    JUCE_NODISCARD Colour withHue (float newHue) const noexcept;
 
     /** Returns a copy of this colour with a different saturation. */
-    Colour withSaturation (float newSaturation) const noexcept;
+    JUCE_NODISCARD Colour withSaturation (float newSaturation) const noexcept;
 
     /** Returns a copy of this colour with a different saturation in the HSL colour space. */
-    Colour withSaturationHSL (float newSaturation) const noexcept;
+    JUCE_NODISCARD Colour withSaturationHSL (float newSaturation) const noexcept;
 
     /** Returns a copy of this colour with a different brightness.
         @see brighter, darker, withMultipliedBrightness
     */
-    Colour withBrightness (float newBrightness) const noexcept;
+    JUCE_NODISCARD Colour withBrightness (float newBrightness) const noexcept;
 
     /** Returns a copy of this colour with a different lightness.
         @see lighter, darker, withMultipliedLightness
     */
-    Colour withLightness (float newLightness) const noexcept;
+    JUCE_NODISCARD Colour withLightness (float newLightness) const noexcept;
 
     /** Returns a copy of this colour with its hue rotated.
         The new colour's hue is ((this->getHue() + amountToRotate) % 1.0)
         @see brighter, darker, withMultipliedBrightness
     */
-    Colour withRotatedHue (float amountToRotate) const noexcept;
+    JUCE_NODISCARD Colour withRotatedHue (float amountToRotate) const noexcept;
 
     /** Returns a copy of this colour with its saturation multiplied by the given value.
         The new colour's saturation is (this->getSaturation() * multiplier)
         (the result is clipped to legal limits).
     */
-    Colour withMultipliedSaturation (float multiplier) const noexcept;
+    JUCE_NODISCARD Colour withMultipliedSaturation (float multiplier) const noexcept;
 
     /** Returns a copy of this colour with its saturation multiplied by the given value.
         The new colour's saturation is (this->getSaturation() * multiplier)
@@ -335,34 +335,34 @@ public:
 
         This will be in the HSL colour space.
     */
-    Colour withMultipliedSaturationHSL (float multiplier) const noexcept;
+    JUCE_NODISCARD Colour withMultipliedSaturationHSL (float multiplier) const noexcept;
 
     /** Returns a copy of this colour with its brightness multiplied by the given value.
         The new colour's brightness is (this->getBrightness() * multiplier)
         (the result is clipped to legal limits).
     */
-    Colour withMultipliedBrightness (float amount) const noexcept;
+    JUCE_NODISCARD Colour withMultipliedBrightness (float amount) const noexcept;
 
     /** Returns a copy of this colour with its lightness multiplied by the given value.
         The new colour's lightness is (this->lightness() * multiplier)
         (the result is clipped to legal limits).
     */
-    Colour withMultipliedLightness (float amount) const noexcept;
+    JUCE_NODISCARD Colour withMultipliedLightness (float amount) const noexcept;
 
     //==============================================================================
     /** Returns a brighter version of this colour.
-        @param amountBrighter   how much brighter to make it - a value from 0 to 1.0 where 0 is
-                                unchanged, and higher values make it brighter
+        @param amountBrighter   how much brighter to make it - a value greater than or equal to 0,
+                                where 0 is unchanged, and higher values make it brighter
         @see withMultipliedBrightness
     */
-    Colour brighter (float amountBrighter = 0.4f) const noexcept;
+    JUCE_NODISCARD Colour brighter (float amountBrighter = 0.4f) const noexcept;
 
     /** Returns a darker version of this colour.
-        @param amountDarker     how much darker to make it - a value from 0 to 1.0 where 0 is
-                                unchanged, and higher values make it darker
+        @param amountDarker     how much darker to make it - a value greater than or equal to 0,
+                                where 0 is unchanged, and higher values make it darker
         @see withMultipliedBrightness
     */
-    Colour darker (float amountDarker = 0.4f) const noexcept;
+    JUCE_NODISCARD Colour darker (float amountDarker = 0.4f) const noexcept;
 
     //==============================================================================
     /** Returns a colour that will be clearly visible against this colour.
@@ -372,7 +372,7 @@ public:
         that's just a little bit lighter; Colours::black.contrasting (1.0f) will
         return white; Colours::white.contrasting (1.0f) will return black, etc.
     */
-    Colour contrasting (float amount = 1.0f) const noexcept;
+    JUCE_NODISCARD Colour contrasting (float amount = 1.0f) const noexcept;
 
     /** Returns a colour that is as close as possible to a target colour whilst
         still being in contrast to this one.
@@ -381,20 +381,20 @@ public:
         nudged up or down so that it differs from the luminosity of this colour
         by at least the amount specified by minLuminosityDiff.
     */
-    Colour contrasting (Colour targetColour, float minLuminosityDiff) const noexcept;
+    JUCE_NODISCARD Colour contrasting (Colour targetColour, float minLuminosityDiff) const noexcept;
 
     /** Returns a colour that contrasts against two colours.
         Looks for a colour that contrasts with both of the colours passed-in.
         Handy for things like choosing a highlight colour in text editors, etc.
     */
-    static Colour contrasting (Colour colour1,
-                               Colour colour2) noexcept;
+    JUCE_NODISCARD static Colour contrasting (Colour colour1,
+                                              Colour colour2) noexcept;
 
     //==============================================================================
     /** Returns an opaque shade of grey.
         @param brightness the level of grey to return - 0 is black, 1.0 is white
     */
-    static Colour greyLevel (float brightness) noexcept;
+    JUCE_NODISCARD static Colour greyLevel (float brightness) noexcept;
 
     //==============================================================================
     /** Returns a stringified version of this colour.
@@ -403,7 +403,7 @@ public:
     String toString() const;
 
     /** Reads the colour from a string that was created with toString(). */
-    static Colour fromString (StringRef encodedColourString);
+    JUCE_NODISCARD static Colour fromString (StringRef encodedColourString);
 
     /** Returns the colour as a hex string in the form RRGGBB or AARRGGBB. */
     String toDisplayString (bool includeAlphaValue) const;
