@@ -2,15 +2,15 @@
   ==============================================================================
 
    This file is part of the JUCE library.
-   Copyright (c) 2020 - Raw Material Software Limited
+   Copyright (c) 2022 - Raw Material Software Limited
 
    JUCE is an open source library subject to commercial or open-source
    licensing.
 
-   By using JUCE, you agree to the terms of both the JUCE 6 End-User License
-   Agreement and JUCE Privacy Policy (both effective as of the 16th June 2020).
+   By using JUCE, you agree to the terms of both the JUCE 7 End-User License
+   Agreement and JUCE Privacy Policy.
 
-   End User License Agreement: www.juce.com/juce-6-licence
+   End User License Agreement: www.juce.com/juce-7-licence
    Privacy Policy: www.juce.com/juce-privacy-policy
 
    Or: You may also use this code under the terms of the GPL v3 (see
@@ -26,6 +26,14 @@
 namespace juce
 {
 
+static double getStepSize (const Slider& slider)
+{
+    const auto interval = slider.getInterval();
+
+    return interval != 0.0 ? interval
+                           : slider.getRange().getLength() * 0.01;
+}
+
 class Slider::Pimpl   : public AsyncUpdater, // this needs to be public otherwise it will cause an
                                              // error when JUCE_DLL_BUILD=1
                         private Value::Listener
@@ -191,9 +199,10 @@ public:
 
             lastCurrentValue = newValue;
 
-            // (need to do this comparison because the Value will use equalsWithSameType to compare
-            // the new and old values, so will generate unwanted change events if the type changes)
-            if (currentValue != newValue)
+            // Need to do this comparison because the Value will use equalsWithSameType to compare
+            // the new and old values, so will generate unwanted change events if the type changes.
+            // Cast to double before comparing, to prevent comparing as another type (e.g. String).
+            if (static_cast<double> (currentValue.getValue()) != newValue)
                 currentValue = newValue;
 
             updateText();
@@ -339,6 +348,9 @@ public:
 
         if (owner.onValueChange != nullptr)
             owner.onValueChange();
+
+        if (auto* handler = owner.getAccessibilityHandler())
+            handler->notifyAccessibilityEvent (AccessibilityEvent::valueChanged);
     }
 
     void sendDragStart()
@@ -370,16 +382,6 @@ public:
             owner.onDragEnd();
     }
 
-    struct DragInProgress
-    {
-        DragInProgress (Pimpl& p)  : owner (p)      { owner.sendDragStart(); }
-        ~DragInProgress()                           { owner.sendDragEnd(); }
-
-        Pimpl& owner;
-
-        JUCE_DECLARE_NON_COPYABLE (DragInProgress)
-    };
-
     void incrementOrDecrement (double delta)
     {
         if (style == IncDecButtons)
@@ -392,7 +394,7 @@ public:
             }
             else
             {
-                DragInProgress drag (*this);
+                ScopedDragNotification drag (owner);
                 setValue (newValue, sendNotificationSync);
             }
         }
@@ -406,9 +408,13 @@ public:
                 setValue (currentValue.getValue(), dontSendNotification);
         }
         else if (value.refersToSameSourceAs (valueMin))
+        {
             setMinValue (valueMin.getValue(), dontSendNotification, true);
+        }
         else if (value.refersToSameSourceAs (valueMax))
+        {
             setMaxValue (valueMax.getValue(), dontSendNotification, true);
+        }
     }
 
     void textChanged()
@@ -417,7 +423,7 @@ public:
 
         if (newValue != static_cast<double> (currentValue.getValue()))
         {
-            DragInProgress drag (*this);
+            ScopedDragNotification drag (owner);
             setValue (newValue, sendNotificationSync);
         }
 
@@ -465,8 +471,10 @@ public:
         if (style != newStyle)
         {
             style = newStyle;
+
             owner.repaint();
             owner.lookAndFeelChanged();
+            owner.invalidateAccessibilityHandler();
         }
     }
 
@@ -588,26 +596,24 @@ public:
             incButton.reset (lf.createSliderButton (owner, true));
             decButton.reset (lf.createSliderButton (owner, false));
 
-            owner.addAndMakeVisible (incButton.get());
-            owner.addAndMakeVisible (decButton.get());
-
-            incButton->onClick = [this] { incrementOrDecrement (normRange.interval); };
-            decButton->onClick = [this] { incrementOrDecrement (-normRange.interval); };
-
-            if (incDecButtonMode != incDecButtonsNotDraggable)
-            {
-                incButton->addMouseListener (&owner, false);
-                decButton->addMouseListener (&owner, false);
-            }
-            else
-            {
-                incButton->setRepeatSpeed (300, 100, 20);
-                decButton->setRepeatSpeed (300, 100, 20);
-            }
-
             auto tooltip = owner.getTooltip();
-            incButton->setTooltip (tooltip);
-            decButton->setTooltip (tooltip);
+
+            auto setupButton = [&] (Button& b, bool isIncrement)
+            {
+                owner.addAndMakeVisible (b);
+                b.onClick = [this, isIncrement] { incrementOrDecrement (isIncrement ? normRange.interval : -normRange.interval); };
+
+                if (incDecButtonMode != incDecButtonsNotDraggable)
+                    b.addMouseListener (&owner, false);
+                else
+                    b.setRepeatSpeed (300, 100, 20);
+
+                b.setTooltip (tooltip);
+                b.setAccessible (false);
+            };
+
+            setupButton (*incButton, true);
+            setupButton (*decButton, false);
         }
         else
         {
@@ -864,7 +870,7 @@ public:
                         popupDisplay->stopTimer();
                 }
 
-                currentDrag.reset (new DragInProgress (*this));
+                currentDrag = std::make_unique<ScopedDragNotification> (owner);
                 mouseDrag (e);
             }
         }
@@ -993,6 +999,38 @@ public:
         popupDisplay.reset();
     }
 
+    bool keyPressed (const KeyPress& key)
+    {
+        if (key.getModifiers().isAnyModifierKeyDown())
+            return false;
+
+        const auto getInterval = [this]
+        {
+            if (auto* accessibility = owner.getAccessibilityHandler())
+                if (auto* valueInterface = accessibility->getValueInterface())
+                    return valueInterface->getRange().getInterval();
+
+            return getStepSize (owner);
+        };
+
+        const auto valueChange = [&]
+        {
+            if (key == KeyPress::rightKey || key == KeyPress::upKey)
+                return getInterval();
+
+            if (key == KeyPress::leftKey || key == KeyPress::downKey)
+                return -getInterval();
+
+            return 0.0;
+        }();
+
+        if (valueChange == 0.0)
+            return false;
+
+        setValue (getValue() + valueChange, sendNotificationSync);
+        return true;
+    }
+
     void showPopupDisplay()
     {
         if (style == IncDecButtons)
@@ -1042,7 +1080,7 @@ public:
     {
         if (canDoubleClickToValue())
         {
-            DragInProgress drag (*this);
+            ScopedDragNotification drag (owner);
             setValue (doubleClickReturnValue, sendNotificationSync);
         }
     }
@@ -1085,7 +1123,7 @@ public:
                     {
                         auto newValue = value + jmax (normRange.interval, std::abs (delta)) * (delta < 0 ? -1.0 : 1.0);
 
-                        DragInProgress drag (*this);
+                        ScopedDragNotification drag (owner);
                         setValue (owner.snapValue (newValue, notDragging), sendNotificationSync);
                     }
                 }
@@ -1210,7 +1248,6 @@ public:
     }
 
     //==============================================================================
-
     void resizeIncDecButtons()
     {
         auto buttonRect = sliderRect;
@@ -1257,7 +1294,7 @@ public:
     int pixelsForFullDragExtent = 250;
     Time lastMouseWheelTime;
     Rectangle<int> sliderRect;
-    std::unique_ptr<DragInProgress> currentDrag;
+    std::unique_ptr<ScopedDragNotification> currentDrag;
 
     TextEntryBoxPosition textBoxPos;
     String textSuffix;
@@ -1357,6 +1394,18 @@ public:
     }
 };
 
+//==============================================================================
+Slider::ScopedDragNotification::ScopedDragNotification (Slider& s)
+    : sliderBeingDragged (s)
+{
+    sliderBeingDragged.pimpl->sendDragStart();
+}
+
+Slider::ScopedDragNotification::~ScopedDragNotification()
+{
+    if (sliderBeingDragged.pimpl != nullptr)
+        sliderBeingDragged.pimpl->sendDragEnd();
+}
 
 //==============================================================================
 Slider::Slider()
@@ -1627,6 +1676,7 @@ void Slider::valueChanged() {}
 void Slider::setPopupMenuEnabled (bool menuEnabled)         { pimpl->menuEnabled = menuEnabled; }
 void Slider::setScrollWheelEnabled (bool enabled)           { pimpl->scrollWheelEnabled = enabled; }
 
+bool Slider::isScrollWheelEnabled() const noexcept          { return pimpl->scrollWheelEnabled; }
 bool Slider::isHorizontal() const noexcept                  { return pimpl->isHorizontal(); }
 bool Slider::isVertical() const noexcept                    { return pimpl->isVertical(); }
 bool Slider::isRotary() const noexcept                      { return pimpl->isRotary(); }
@@ -1651,6 +1701,9 @@ void Slider::mouseExit (const MouseEvent&)      { pimpl->mouseExit(); }
 // it is shown when dragging the mouse over a slider and releasing
 void Slider::mouseEnter (const MouseEvent&)     { pimpl->mouseMove(); }
 
+/** @internal */
+bool Slider::keyPressed (const KeyPress& k)     { return pimpl->keyPressed (k); }
+
 void Slider::modifierKeysChanged (const ModifierKeys& modifiers)
 {
     if (isEnabled())
@@ -1675,4 +1728,75 @@ void Slider::mouseWheelMove (const MouseEvent& e, const MouseWheelDetails& wheel
         Component::mouseWheelMove (e, wheel);
 }
 
+//==============================================================================
+class SliderAccessibilityHandler  : public AccessibilityHandler
+{
+public:
+    explicit SliderAccessibilityHandler (Slider& sliderToWrap)
+        : AccessibilityHandler (sliderToWrap,
+                                AccessibilityRole::slider,
+                                AccessibilityActions{},
+                                AccessibilityHandler::Interfaces { std::make_unique<ValueInterface> (sliderToWrap) }),
+          slider (sliderToWrap)
+    {
+    }
+
+    String getHelp() const override   { return slider.getTooltip(); }
+
+private:
+    class ValueInterface  : public AccessibilityValueInterface
+    {
+    public:
+        explicit ValueInterface (Slider& sliderToWrap)
+            : slider (sliderToWrap),
+              useMaxValue (slider.isTwoValue())
+        {
+        }
+
+        bool isReadOnly() const override  { return false; }
+
+        double getCurrentValue() const override
+        {
+            return useMaxValue ? slider.getMaximum()
+                               : slider.getValue();
+        }
+
+        void setValue (double newValue) override
+        {
+            Slider::ScopedDragNotification drag (slider);
+
+            if (useMaxValue)
+                slider.setMaxValue (newValue, sendNotificationSync);
+            else
+                slider.setValue (newValue, sendNotificationSync);
+        }
+
+        String getCurrentValueAsString() const override          { return slider.getTextFromValue (getCurrentValue()); }
+        void setValueAsString (const String& newValue) override  { setValue (slider.getValueFromText (newValue)); }
+
+        AccessibleValueRange getRange() const override
+        {
+            return { { slider.getMinimum(), slider.getMaximum() },
+                     getStepSize (slider) };
+        }
+
+    private:
+        Slider& slider;
+        const bool useMaxValue;
+
+        //==============================================================================
+        JUCE_DECLARE_NON_COPYABLE_WITH_LEAK_DETECTOR (ValueInterface)
+    };
+
+    Slider& slider;
+
+    //==============================================================================
+    JUCE_DECLARE_NON_COPYABLE_WITH_LEAK_DETECTOR (SliderAccessibilityHandler)
+};
+
+std::unique_ptr<AccessibilityHandler> Slider::createAccessibilityHandler()
+{
+    return std::make_unique<SliderAccessibilityHandler> (*this);
+}
+
 } // namespace juce
