@@ -3,7 +3,7 @@
 
    This file is part of the Water library.
    Copyright (c) 2016 ROLI Ltd.
-   Copyright (C) 2017 Filipe Coelho <falktx@falktx.com>
+   Copyright (C) 2017-2022 Filipe Coelho <falktx@falktx.com>
 
    Permission is granted to use this software under the terms of the ISC license
    http://www.isc.org/downloads/software-support-policy/isc-license/
@@ -126,7 +126,7 @@ private:
     //==============================================================================
     Atomic<int> refCount;
 
-    CARLA_DECLARE_NON_COPY_CLASS (ReferenceCountedObject)
+    CARLA_DECLARE_NON_COPYABLE (ReferenceCountedObject)
 };
 
 
@@ -196,7 +196,7 @@ private:
     //==============================================================================
     int refCount;
 
-    CARLA_DECLARE_NON_COPY_CLASS (SingleThreadedReferenceCountedObject)
+    CARLA_DECLARE_NON_COPYABLE (SingleThreadedReferenceCountedObject)
 };
 
 
@@ -301,22 +301,6 @@ public:
         return *this;
     }
 
-   #if WATER_COMPILER_SUPPORTS_MOVE_SEMANTICS
-    /** Takes-over the object from another pointer. */
-    ReferenceCountedObjectPtr (ReferenceCountedObjectPtr&& other) noexcept
-        : referencedObject (other.referencedObject)
-    {
-        other.referencedObject = nullptr;
-    }
-
-    /** Takes-over the object from another pointer. */
-    ReferenceCountedObjectPtr& operator= (ReferenceCountedObjectPtr&& other)
-    {
-        std::swap (referencedObject, other.referencedObject);
-        return *this;
-    }
-   #endif
-
     /** Destructor.
         This will decrement the object's reference-count, which will cause the
         object to be deleted when the ref-count hits zero.
