@@ -56,10 +56,6 @@ public:
     /** Creates a copy of another string. */
     String (const String& other) noexcept;
 
-   #if WATER_COMPILER_SUPPORTS_MOVE_SEMANTICS
-    String (String&& other) noexcept;
-   #endif
-
     /** Creates a string from a zero-terminated ascii text string.
 
         The string passed-in must not contain any characters with a value above 127, because
@@ -139,10 +135,6 @@ public:
     /** Replaces this string's contents with another string. */
     String& operator= (const String& other) noexcept;
 
-   #if WATER_COMPILER_SUPPORTS_MOVE_SEMANTICS
-    String& operator= (String&& other) noexcept;
-   #endif
-
     /** Appends another string at the end of this one. */
     String& operator+= (const String& stringToAppend);
     /** Appends another string at the end of this one. */
@@ -1005,6 +997,11 @@ public:
     */
     CharPointer_UTF8 toUTF8() const;
 
+   #ifdef CARLA_OS_WIN
+    /** Convert string to UTF-16, Windows only */
+    std::wstring toUTF16() const;
+   #endif
+
     /** Returns a pointer to a UTF-8 version of this string.
 
         Because it returns a reference to the string's internal data, the pointer
