@@ -28,7 +28,7 @@
     nullptr, nullptr, nullptr, nullptr, nullptr, \
     nullptr, nullptr
 #define DESCFUNCS_WITHOUTCV \
-    DESCFUNCS_WITHCV, 0, 0, nullptr, nullptr
+    DESCFUNCS_WITHCV, 0, 0, nullptr, nullptr, 0, 0
 
 static const NativePluginDescriptor sNativePluginDescriptors[] = {
 
@@ -85,7 +85,8 @@ static const NativePluginDescriptor sNativePluginDescriptors[] = {
 },
 {
     /* category  */ NATIVE_PLUGIN_CATEGORY_UTILITY,
-    /* hints     */ NATIVE_PLUGIN_IS_RTSAFE,
+    /* hints     */ static_cast<NativePluginHints>(NATIVE_PLUGIN_IS_RTSAFE
+                                                  |NATIVE_PLUGIN_USES_CONTROL_VOLTAGE),
     /* supports  */ NATIVE_PLUGIN_SUPPORTS_NOTHING,
     /* audioIns  */ 0,
     /* audioOuts */ 1,
@@ -101,7 +102,9 @@ static const NativePluginDescriptor sNativePluginDescriptors[] = {
     /* cvIns      */ 1,
     /* cvOuts     */ 0,
     /* bufnamefn  */ nullptr,
-    /* bufrangefn */ nullptr
+    /* bufrangefn */ nullptr,
+    /* ui_width   */ 0,
+    /* ui_height  */ 0
 },
 {
     /* category  */ NATIVE_PLUGIN_CATEGORY_UTILITY,
@@ -201,7 +204,8 @@ static const NativePluginDescriptor sNativePluginDescriptors[] = {
 },
 {
     /* category  */ NATIVE_PLUGIN_CATEGORY_UTILITY,
-    /* hints     */ NATIVE_PLUGIN_IS_RTSAFE,
+    /* hints     */ static_cast<NativePluginHints>(NATIVE_PLUGIN_IS_RTSAFE
+                                                  |NATIVE_PLUGIN_USES_CONTROL_VOLTAGE),
     /* supports  */ NATIVE_PLUGIN_SUPPORTS_ALL_SOUND_OFF,
     /* audioIns  */ 0,
     /* audioOuts */ 0,
@@ -217,7 +221,9 @@ static const NativePluginDescriptor sNativePluginDescriptors[] = {
     /* cvIns      */ 0,
     /* cvOuts     */ 3,
     /* bufnamefn  */ nullptr,
-    /* bufrangefn */ nullptr
+    /* bufrangefn */ nullptr,
+    /* ui_width   */ 0,
+    /* ui_height  */ 0
 },
 {
     /* category  */ NATIVE_PLUGIN_CATEGORY_UTILITY,
@@ -506,7 +512,9 @@ static const NativePluginDescriptor sNativePluginDescriptors[] = {
     /* cvIns      */ 5,
     /* cvOuts     */ 5,
     /* bufnamefn  */ nullptr,
-    /* bufrangefn */ nullptr
+    /* bufrangefn */ nullptr,
+    /* ui_width   */ 0,
+    /* ui_height  */ 0
 },
 #endif
 
@@ -550,6 +558,23 @@ static const NativePluginDescriptor sNativePluginDescriptors[] = {
     /* copyright */ "GNU GPL v2+",
     DESCFUNCS_WITHOUTCV
 },
+{
+    /* category  */ NATIVE_PLUGIN_CATEGORY_UTILITY,
+    /* hints     */ static_cast<NativePluginHints>(NATIVE_PLUGIN_IS_RTSAFE
+                                                  |NATIVE_PLUGIN_HAS_UI),
+    /* supports  */ NATIVE_PLUGIN_SUPPORTS_NOTHING,
+    /* audioIns  */ 0,
+    /* audioOuts */ 0,
+    /* midiIns   */ 1,
+    /* midiOuts  */ 1,
+    /* paramIns  */ 2,
+    /* paramOuts */ 2,
+    /* name      */ "XY Controller",
+    /* label     */ "xycontroller",
+    /* maker     */ "falkTX",
+    /* copyright */ "GNU GPL v2+",
+    DESCFUNCS_WITHOUTCV
+},
 #endif
 
 #ifdef HAVE_EXTERNAL_PLUGINS
