@@ -428,6 +428,14 @@ public:
         CARLA_SAFE_ASSERT_RETURN(fHostWindow != 0,);
 
         XStoreName(fDisplay, fHostWindow, title);
+
+        const Atom _nwn = XInternAtom(fDisplay, "_NET_WM_NAME", False);
+        const Atom utf8 = XInternAtom(fDisplay, "UTF8_STRING", True);
+
+        XChangeProperty(fDisplay, fHostWindow, _nwn, utf8, 8,
+                        PropModeReplace,
+                        (const uchar*)(title),
+                        (int)strlen(title));
     }
 
     void setTransientWinId(const uintptr_t winId) override
@@ -1472,7 +1480,7 @@ bool CarlaPluginUI::tryTransientWinIdMatch(const uintptr_t pid, const char* cons
 CarlaPluginUI* CarlaPluginUI::newX11(Callback* const cb,
                                      const uintptr_t parentId,
                                      const bool isStandalone,
-                                     const bool isResizable, 
+                                     const bool isResizable,
                                      const bool isLV2)
 {
     return new X11PluginUI(cb, parentId, isStandalone, isResizable, isLV2);
